/*
 * kick.h
 *
 *  Created on: 21.05.2014
 *      Author: AndreR
 */

#ifndef KICK_H_
#define KICK_H_

#include "ch.h"
#include "util/config.h"
#include "commands.h"

#define KICKER_DEVICE_STRAIGHT	0
#define KICKER_DEVICE_CHIP		1

#define KICK_EVENT_QUEUE_SIZE 4

typedef struct PACKED _KickerInternalConfig
{
	float maxVoltage;		// [V]
	float chgHysteresis;	// [V]
	uint32_t cooldown;		// [ms]
	float irThreshold;		// [V]
	float kickVoltageLimit; // ignore kick cmds below kickVoltageLimit*curVoltage

	float kickOffset;
	float kickFactor;
	float kickMax;

	float chipOffset;
	float chipFactor;
	float chipMax;
} KickerInternalConfig;

typedef struct _KickGlobal
{
	uint16_t irDiff;
	uint8_t interrupted;

	systime_t lastBarrierClearTime;
	uint16_t barrierDamaged;

	uint16_t state;

	KickerInternalConfig config;
	ConfigFile* pConfigFile;

	struct _armed
	{
		uint8_t device;
		float speed;
		float rpm;
	} armed;

	mailbox_t eventQueue;
	msg_t eventQueueData[KICK_EVENT_QUEUE_SIZE];
} KickGlobal;

extern KickGlobal kick;

void	KickInit();
void	KickTask(void* params);
void	KickFire(float duration, uint8_t device);
void	KickFireSpeed(float speed, uint8_t device);
void	KickArm(float duration, uint8_t device);
void	KickArmSpeed(float fireDuration, uint8_t device);
void	KickDisarm();
void	KickDischarge();
void	KickSetIRConfig(uint16_t irThreshold);
void	KickSetFireConfig(float maxCap, uint32_t cooldown, float kickLimit);
void	KickSetChgConfig(float hyst);

#endif /* KICK_H_ */
