/*
 * spi2.h
 *
 *  Created on: 19.01.2014
 *      Author: AndreR
 */

#ifndef SPI2_H_
#define SPI2_H_

#include "util/spi_sync.h"
#include "util/lag_element.h"
#include "ch.h"
#include "log_msgs.h"

typedef void(*SPI2Callback)();

#define SPI2_EVENT_QUEUE_SIZE 4

#define SPI2_IMU_BUF_SIZE 10

#define SPI2_ACC_UNIT_TO_MS2 (5.9875488281250003e-4f)
#define SPI2_GYR_UNIT_TO_RADS (0.001221730476396f)

typedef struct _SPI2Global
{
	SPISync bus;
	SPISyncSlave slaveAcc;
	SPISyncSlave slaveGyr;

	SPI2Callback gyrCb;
	SPI2Callback accCb;

	mailbox_t eventQueue;
	msg_t eventQueueData[SPI2_EVENT_QUEUE_SIZE];

	uint32_t lastGyrTime;
	uint32_t avgGyrPeriod;

	uint32_t lastAccTime;
	uint32_t avgAccPeriod;

	int16_t gyrRaw[3];
	int16_t accRaw[3];

	float gyr[3];
	float acc[3];

	LogRawI16XYZ logGyr[SPI2_IMU_BUF_SIZE];
	uint16_t logGyrUsed;
	LogRawI16XYZ logAcc[SPI2_IMU_BUF_SIZE];
	uint16_t logAccUsed;

#ifdef DEBUG
	DMA_Stream_TypeDef* pTxDma;
	DMA_Stream_TypeDef* pRxDma;
	SPI_TypeDef* pSPI;
	DMA_TypeDef* pDMA;
#endif
} SPI2Global;

extern SPI2Global spi2;

void	SPI2Init();
void	SPI2Task(void* params);
void	SPI2SetCallbacks(SPI2Callback gyrRaw, SPI2Callback accRaw);

#endif /* SPI2_H_ */
