/*
 * optical.c
 *
 *  Created on: 17.10.2015
 *      Author: AndreR
 */

#include "optical.h"
#include <stdio.h>

static struct _handles
{
	GHandle hDeltaX[2];
	GHandle hDeltaY[2];
	GHandle hSQUAL[2];
	GHandle hPixelSum[2];
	GHandle hMaxPixel[2];
	GHandle hMinPixel[2];
	GHandle hShutter[2];
	GHandle hFramePeriod[2];

	GHandle speed[3];
} handles;

static GHandle createLabel(coord_t x, coord_t y, coord_t w, coord_t h, GHandle parent, const char* pText)
{
	GWidgetInit deviceLabelInit = { { x, y, w, h, TRUE, parent }, pText, 0, 0, 0 };
	return gwinLabelCreate(0, &deviceLabelInit);
}

GHandle OpticalCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Optical", 0, 0, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    font_t curFont = gwinGetDefaultFont();
    gwinSetDefaultFont(gdispOpenFont("DejaVuSans10"));

    createLabel(5, 5, 70, 15, hTop, "DeltaX:");
    createLabel(5, 20, 70, 15, hTop, "DeltaY:");
    createLabel(5, 35, 70, 15, hTop, "SQUAL:");
    createLabel(5, 50, 70, 15, hTop, "PixelSum:");
    createLabel(5, 65, 70, 15, hTop, "Max Pixel:");
    createLabel(5, 80, 70, 15, hTop, "Min Pixel:");
    createLabel(5, 95, 70, 15, hTop, "Shutter:");
    createLabel(5, 125, 70, 15, hTop, "Frame Rate:");

    createLabel(5, 155, 70, 15, hTop, "X:");
    createLabel(5, 170, 70, 15, hTop, "Y:");
    createLabel(5, 185, 70, 15, hTop, "W:");

    for(uint16_t i = 0; i < 2; i++)
    {
		handles.hDeltaX[i] = createLabel(80+i*80, 5, 80, 15, hTop, "dev");
		handles.hDeltaY[i] = createLabel(80+i*80, 20, 80, 15, hTop, "?");
		handles.hSQUAL[i] = createLabel(80+i*80, 35, 80, 15, hTop, "prod");
		handles.hPixelSum[i] = createLabel(80+i*80, 50, 80, 15, hTop, "012345");
		handles.hMaxPixel[i] = createLabel(80+i*80, 65, 80, 15, hTop, "42 MB");
		handles.hMinPixel[i] = createLabel(80+i*80, 80, 80, 15, hTop, "43 MB");
		handles.hShutter[i] = createLabel(80+i*80, 95, 80, 15, hTop, "16k");
		handles.hFramePeriod[i] = createLabel(80+i*80, 125, 80, 15, hTop, "N/A");
    }

    handles.speed[0] = createLabel(80, 155, 80, 15, hTop, "0");
    handles.speed[1] = createLabel(80, 170, 80, 15, hTop, "0");
    handles.speed[2] = createLabel(80, 185, 80, 15, hTop, "0");

    gwinSetDefaultFont(curFont);

    return hTop;
}

void OpticalSpeedUpdate(float* pSpeed)
{
	static char sp[3][10];

	for(uint8_t i = 0; i < 3; i++)
	{
		snprintf(sp[i], 10, "%.5f", pSpeed[i]);
		gwinSetText(handles.speed[i], sp[i], FALSE);
	}
}

void OpticalUpdate(ADNS9800Motion* pMot, uint8_t pos)
{
	static char deltaX[2][10];
	static char deltaY[2][10];
	static char squal[2][10];
	static char pixelSum[2][10];
	static char minPixel[2][10];
	static char maxPixel[2][10];
	static char shutter[2][10];
	static char frame[2][10];

//	snprintf(deltaX[pos], 10, "%hd", pMot->deltaX);
//	gwinSetText(handles.hDeltaX[pos], deltaX[pos], FALSE);
//
//	snprintf(deltaY[pos], 10, "%hd", pMot->deltaY);
//	gwinSetText(handles.hDeltaY[pos], deltaY[pos], FALSE);

	snprintf(deltaX[pos], 10, "%.3f", pMot->speed[0]);
	gwinSetText(handles.hDeltaX[pos], deltaX[pos], FALSE);

	snprintf(deltaY[pos], 10, "%.3f", pMot->speed[1]);
	gwinSetText(handles.hDeltaY[pos], deltaY[pos], FALSE);

	snprintf(squal[pos], 10, "%hu", (uint16_t)pMot->squal);
	gwinSetText(handles.hSQUAL[pos], squal[pos], FALSE);

	snprintf(pixelSum[pos], 10, "%hu", (uint16_t)pMot->pixelSum);
	gwinSetText(handles.hPixelSum[pos], pixelSum[pos], FALSE);

	snprintf(minPixel[pos], 10, "%hu", (uint16_t)pMot->minPixel);
	gwinSetText(handles.hMinPixel[pos], minPixel[pos], FALSE);

	snprintf(maxPixel[pos], 10, "%hu", (uint16_t)pMot->maxPixel);
	gwinSetText(handles.hMaxPixel[pos], maxPixel[pos], FALSE);

	uint32_t shutterNs = pMot->shutter*20;
	snprintf(shutter[pos], 10, "%u", shutterNs);
	gwinSetText(handles.hShutter[pos], shutter[pos], FALSE);

	uint32_t frameRate = 50000000/pMot->framePeriod;
	snprintf(frame[pos], 10, "%u", frameRate);
	gwinSetText(handles.hFramePeriod[pos], frame[pos], FALSE);
}
