/*
 * prog_source_usb.c
 *
 *  Created on: 02.04.2015
 *      Author: AndreR
 */

#include "prog_source_usb.h"
#include "util/crc.h"
#include "util/console.h"

#include <string.h>

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size);
static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
static void sizeFunc(uint8_t procId);
static void open();
static void close();

ProgSourceUSB progSourceUSB = {.usb = {&dataFunc, &sizeFunc, &crcFunc, &open, &close, 100, PROG_SOURCE_USB_BUF}};

static const char* mainFilename = "0:/main2016.bin";

void ProgSourceUSBInit()
{
	ProgStoreAddSource(&progSourceUSB.usb);
}

static void open()
{
	f_open(&progSourceUSB.mainFile, (TCHAR*)mainFilename, FA_READ | FA_OPEN_EXISTING);
}

static void close()
{
	f_close(&progSourceUSB.mainFile);
}

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size)
{
	if(procId != PROG_STORE_ID_MAIN2016)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	memset(progSourceUSB.buf, 0xFF, PROG_SOURCE_USB_BUF);

	UINT bytesRead;
	FRESULT fresult;
	fresult = f_lseek(&progSourceUSB.mainFile, offset);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	fresult = f_read(&progSourceUSB.mainFile, progSourceUSB.buf, PROG_SOURCE_USB_BUF, &bytesRead);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	ProgStoreProvideData(procId, offset, progSourceUSB.buf, size);
}

static void sizeFunc(uint8_t procId)
{
	if(procId != PROG_STORE_ID_MAIN2016)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	FRESULT fresult;
	FILINFO info;
	memset(&info, 0, sizeof(FILINFO));

	fresult = f_stat(mainFilename, &info);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	ProgStoreProvideSize(procId, info.fsize);
}

static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr)
{
	if(procId != PROG_STORE_ID_MAIN2016)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	UINT bytesRead;
	FRESULT fresult;
	fresult = f_lseek(&progSourceUSB.mainFile, startAddr);
	if(fresult)
	{
		ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
		return;
	}

	uint32_t addr = 0;
	uint32_t size = endAddr - startAddr;

	uint32_t crc = CRC32StartChecksumAlt();

	while(addr < size)
	{
		uint32_t readSize = PROG_SOURCE_USB_BUF;

		if(addr + PROG_SOURCE_USB_BUF > size)
			readSize = size-addr;

		memset(progSourceUSB.buf, 0xFF, PROG_SOURCE_USB_BUF);

		fresult = f_read(&progSourceUSB.mainFile, progSourceUSB.buf, readSize, &bytesRead);
		if(fresult)
		{
			ProgStoreProvideError(procId, PROG_STORE_ERROR_NO_MEDIA);
			break;
		}

		crc = CRC32FeedChecksumAlt(crc, progSourceUSB.buf, readSize);

		addr += readSize;
	}

	crc = CRC32StopChecksumAlt(crc);

	ProgStoreProvideCRC32(procId, startAddr, endAddr, crc);
}
