/*
 * prog_store_impl.c
 *
 *  Created on: 13.04.2016
 *      Author: AndreR
 */

#include "prog_store_impl.h"
#include "util/prog_store.h"
#include "intercom_constants.h"
#include "string.h"
#include "util/crc.h"
#include "util/flash.h"
#include "hal/ext_flash.h"

#define PROG_SIZE_MAIN (384*1024)

#define PROG_BASE_ADDR 0x08080000

#define PROG_INFO_ADDR	(PROG_BASE_ADDR+0)
#define PROG_MAIN_ADDR	(PROG_INFO_ADDR+4096)

const uint8_t* pProgs[] = {
	(const uint8_t*)PROG_MAIN_ADDR,
};

const ProgUpgradeInfo* pInfoFlash = (ProgUpgradeInfo*)PROG_INFO_ADDR;

void ProgStoreImplInit()
{
	ProgStoreAddTarget(INTERCOM_ADDRESS_MAIN, PROG_SIZE_MAIN, PROG_STORE_ID_MAIN2016);
}

void ProgStoreImplWriteData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size)
{
	FlashProgram((uint32_t)(pProgs[procId]+offset), (uint32_t*)pData, (size+3)/4);
}

void ProgStoreImplGetUpgradeInfo(uint8_t procId, ProgUpgradeInfo* pInfo)
{
	memcpy(pInfo, &pInfoFlash[procId], sizeof(ProgUpgradeInfo));
}

void ProgStoreImplSetUpgradeInfo(uint8_t procId, const ProgUpgradeInfo* pInfo)
{
	FlashProgram((uint32_t)(&pInfoFlash[procId]), (uint32_t*)pInfo, sizeof(ProgUpgradeInfo)/4);
}

void ProgStoreImplReadData(uint8_t procId, uint32_t offset, uint8_t* pBuf, uint32_t size)
{
	memcpy(pBuf, pProgs[procId]+offset, size);
}

uint32_t ProgStoreImplGetChecksum(uint8_t procId, uint32_t offset, uint32_t size)
{
	return CRC32CalcChecksumAlt(pProgs[procId]+offset, size);
}

void ProgStoreImplEraseStorage()
{
	// erase upper 512kB of program memory
	// this will stall execution for around 4s
	FlashErase(0x08080000, 0x08100000);
}
