/*
 * test.c
 *
 *  Created on: 13.12.2015
 *      Author: AndreR <andre@ryll.cc>
 */

#include "test.h"
#include "util/intercom.h"
#include "util/intercom_reliable.h"
#include "gui/test_drive.h"
#include "gui/test_dribbler.h"
#include "util/console.h"

Test test;

static void testCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case IC_TEST_PROGRESS:
		{
			TestProgress* pProg = (TestProgress*)pData;
			if(pProg->motorId == 4)
			{
				TestDribblerProgress(pProg);
			}

			if(pProg->motorId > 3 || pProg->testId < IC_TEST_MOT_HALL || pProg->testId > IC_TEST_MOT_ENC)
				return;

			TestDriveProgress(pProg);
		}
		break;
		case IC_TEST_MOT_POWER:
		{
			TestMotorPowerUnitResult* pResult = (TestMotorPowerUnitResult*)pData;

			if(pResult->motorId == 4)
				TestDribblerPowerUnitResult(pResult);
			else
				TestDrivePowerUnitResult(pResult);

			chBSemSignal(&test.testFinished);
		}
		break;
		case IC_TEST_MOT_HALL:
		{
			TestMotorHallResult* pResult = (TestMotorHallResult*)pData;

			if(pResult->motorId == 4)
				TestDribblerHallResult(pResult);
			else
				TestDriveHallResult(pResult);

			chBSemSignal(&test.testFinished);
		}
		break;
		case IC_TEST_MOT_DYN:
		{
			TestMotorDynamicsResult* pResult = (TestMotorDynamicsResult*)pData;

			if(pResult->motorId == 4)
				TestDribblerDynamicResult(pResult);
			else
				TestDriveDynamicResult(pResult);

			chBSemSignal(&test.testFinished);
		}
		break;
		case IC_TEST_MOT_ENC:
		{
			TestMotorEncoderResult* pResult = (TestMotorEncoderResult*)pData;
			TestDriveEncoderResult(pResult);

			chBSemSignal(&test.testFinished);
		}
		break;
	}
}

void TestInit()
{
	chMBObjectInit(&test.eventQueue, test.eventQueueData, TEST_EVENT_QUEUE_SIZE);
	chBSemObjectInit(&test.testFinished, 0);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &testCallback;
	client.section = INTERCOM_SECTION_TEST;

	IntercomAddClient(client);
}

void TestScheduleMotorTest(uint8_t testId, uint8_t motorId)
{
	msg_t event = motorId;
	event <<= 8;
	event |= testId;

	chMBPost(&test.eventQueue, event, TIME_IMMEDIATE);
}

void TestTask(void* params)
{
	(void)params;

	msg_t event;

	chRegSetThreadName("Test");

	while(1)
	{
		if(chMBFetch(&test.eventQueue, &event, TIME_INFINITE) != MSG_OK)
			continue;

		uint8_t testId = event & 0xFF;
		uint8_t motorId = event >> 8;

		chBSemReset(&test.testFinished, 1);

		IntercomHeader iHeader;
		if(motorId == 0 || motorId == 3)
			iHeader.address = INTERCOM_ADDRESS_RIGHT;

		if(motorId == 1 || motorId == 2)
			iHeader.address = INTERCOM_ADDRESS_LEFT;

		if(motorId == 4)
			iHeader.address = INTERCOM_ADDRESS_KD;

		iHeader.section = INTERCOM_SECTION_TEST;
		iHeader.command = IC_TEST_START;

		TestStart start;
		start.testId = testId;
		start.motorId = motorId;

		IntercomReliableSend(&iHeader, sizeof(TestStart), &start);

		chBSemWait(&test.testFinished);
	}
}
