/*
 * network.h
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#ifndef __NETWORK_H__
#define __NETWORK_H__

#include "util/nrf24_ex.h"
#include "util/flash_fs.h"
#include "util/fifo_lin.h"
#include "util/config.h"
#include "util/ema_filter.h"

#include "commands.h"
#include "intercom_constants.h"

#include "ch.h"

// Configuration
#define NETWORK_TX_PROCESSING_SIZE	1000
#define NETWORK_RX_PROCESSING_SIZE	1000
#define NETWORK_PRINT_SIZE 100
#define NETWORK_RELIABLE_BUF_SIZE 400

#define NETWORK_REL_EVENT_SIZE 4

typedef struct _Network
{
	uint8_t txProcessingBuf[NETWORK_TX_PROCESSING_SIZE];
	uint8_t rxProcessingBuf[NETWORK_RX_PROCESSING_SIZE];

	uint8_t netPrint[NETWORK_PRINT_SIZE];

	uint8_t reliableBuf[NETWORK_RELIABLE_BUF_SIZE];
	FifoLin reliableCmds;
	uint16_t seq;
	msg_t relEventsData[NETWORK_REL_EVENT_SIZE];
	mailbox_t relEvents;

	binary_semaphore_t timerSem;
	mutex_t sendPacketMutex;
	mutex_t printMutex;

	NRF24Queue queue;
	NRF24Ex nrf24;

	uint32_t lastUSec;

	SystemMatchCtrl lastMatchCtrlCmd;
	uint32_t matchCtrlTime;
	uint8_t matchCtrlUpdated;

	ConfigNetwork config;
	ConfigFile* pNetworkConfig;

	NRF24ReadWriteDelay rwDelay;
	ConfigFile* pRwDelayConfig;

	EMAFilter avgRxPeriod;

	uint8_t linkDisabled;
} Network;

extern Network network;

void	NetworkInit();
void	NetworkTask(void* params);
void	NetworkReliableTask(void* params);
int16_t NetworkSendPacket(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSendPacketRaw(void* _pData, uint16_t dataLength);
int16_t NetworkSendPacketReliable(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSaveConfig();

void	NetworkImplInit();
void	NetworkImplStartTimeout(uint32_t microSeconds);
void	NetworkImplProcessPacket(PacketHeader* pHeader, uint8_t* pBuf, uint16_t dataLength);

#endif /* __NETWORK_H__ */
